@@ -6,9 +6,9 @@ from packaging.utils import canonicalize_name
 
 from poetry.core.constraints.version.exceptions import ParseConstraintError
 from poetry.core.packages.dependency import Dependency
-from poetry.core.version.markers import InvalidMarker
+from poetry.core.version.markers import InvalidMarkerError
 from poetry.core.version.markers import parse_marker
-from poetry.core.version.requirements import InvalidRequirement
+from poetry.core.version.requirements import InvalidRequirementError
 
 
 @pytest.mark.parametrize(
@@ -24,12 +24,23 @@ from poetry.core.version.requirements import InvalidRequirement
         "^1.0.0-1",
     ],
 )
-@pytest.mark.parametrize("allows_prereleases", [False, True])
+@pytest.mark.parametrize("allows_prereleases", [None, False, True])
 def test_allows_prerelease(constraint: str, allows_prereleases: bool) -> None:
     dependency = Dependency("A", constraint, allows_prereleases=allows_prereleases)
     assert dependency.allows_prereleases() == allows_prereleases
 
 
+def test_python_versions_are_made_precise() -> None:
+    dependency = Dependency("Django", "^1.23")
+    dependency.python_versions = ">3.6,<=3.10"
+
+    assert (
+        str(dependency.marker)
+        == 'python_full_version > "3.6.0" and python_full_version <= "3.10.0"'
+    )
+    assert str(dependency.python_constraint) == ">3.6,<=3.10"
+
+
 def test_to_pep_508() -> None:
     dependency = Dependency("Django", "^1.23")
 
@@ -41,8 +52,7 @@ def test_to_pep_508() -> None:
 
     result = dependency.to_pep_508()
     assert (
-        result
-        == "Django (>=1.23,<2.0) ; "
+        result == "Django (>=1.23,<2.0) ; "
         'python_version >= "2.7" and python_version < "2.8" '
         'or python_version >= "3.6" and python_version < "4.0"'
     )
@@ -57,7 +67,7 @@ def test_to_pep_508_wilcard() -> None:
 
 def test_to_pep_508_in_extras() -> None:
     dependency = Dependency("Django", "^1.23")
-    dependency.in_extras.append(canonicalize_name("foo"))
+    dependency._in_extras = [canonicalize_name("foo")]
 
     result = dependency.to_pep_508()
     assert result == 'Django (>=1.23,<2.0) ; extra == "foo"'
@@ -65,7 +75,7 @@ def test_to_pep_508_in_extras() -> None:
     result = dependency.to_pep_508(with_extras=False)
     assert result == "Django (>=1.23,<2.0)"
 
-    dependency.in_extras.append(canonicalize_name("bar"))
+    dependency._in_extras = [canonicalize_name("foo"), canonicalize_name("bar")]
 
     result = dependency.to_pep_508()
     assert result == 'Django (>=1.23,<2.0) ; extra == "foo" or extra == "bar"'
@@ -74,8 +84,7 @@ def test_to_pep_508_in_extras() -> None:
 
     result = dependency.to_pep_508()
     assert (
-        result
-        == "Django (>=1.23,<2.0) ; "
+        result == "Django (>=1.23,<2.0) ; "
         "("
         'python_version >= "2.7" and python_version < "2.8" '
         'or python_version >= "3.6" and python_version < "4.0"'
@@ -85,8 +94,7 @@ def test_to_pep_508_in_extras() -> None:
 
     result = dependency.to_pep_508(with_extras=False)
     assert (
-        result
-        == "Django (>=1.23,<2.0) ; "
+        result == "Django (>=1.23,<2.0) ; "
         'python_version >= "2.7" and python_version < "2.8" '
         'or python_version >= "3.6" and python_version < "4.0"'
     )
@@ -137,6 +145,7 @@ def test_to_pep_508_with_patch_python_version(
     expected = f"Django (>=1.23,<2.0) ; {marker}"
 
     assert dependency.to_pep_508() == expected
+    assert dependency.to_pep_508(resolved=True) == expected
     assert str(dependency.marker) == marker
 
 
@@ -194,7 +203,7 @@ def test_to_pep_508_combination() -> None:
     ],
 )
 def test_to_pep_508_with_invalid_marker(requirement: str) -> None:
-    with pytest.raises(InvalidMarker):
+    with pytest.raises(InvalidMarkerError):
         _ = Dependency.create_from_pep_508(requirement)
 
 
@@ -205,7 +214,7 @@ def test_to_pep_508_with_invalid_marker(requirement: str) -> None:
     ],
 )
 def test_to_pep_508_with_invalid_requirement(requirement: str) -> None:
-    with pytest.raises(InvalidRequirement):
+    with pytest.raises(InvalidRequirementError):
         _ = Dependency.create_from_pep_508(requirement)
 
 
@@ -302,7 +311,6 @@ def test_with_constraint() -> None:
         'python_version >= "3.7" and python_version < "4.0"'
     )
     dependency.python_versions = "^3.6"
-    dependency.transitive_python_versions = "^3.7"
 
     new = dependency.with_constraint("^1.2.6")
 
@@ -315,7 +323,6 @@ def test_with_constraint() -> None:
     assert new.marker == dependency.marker
     assert new.transitive_marker == dependency.transitive_marker
     assert new.python_constraint == dependency.python_constraint
-    assert new.transitive_python_constraint == dependency.transitive_python_constraint
 
 
 @pytest.mark.parametrize(
@@ -355,6 +362,11 @@ def test_create_from_pep_508_url_with_activated_extras() -> None:
     assert dependency.extras == {"fred", "bar"}
 
 
+def test_create_from_pep_508_starting_with_digit() -> None:
+    dependency = Dependency.create_from_pep_508("2captcha-python")
+    assert dependency.name == "2captcha-python"
+
+
 @pytest.mark.parametrize(
     "dependency1, dependency2, expected",
     [
@@ -388,7 +400,6 @@ def test_eq(dependency1: Dependency, dependency2: Dependency, expected: bool) ->
     [
         ("constraint", "2.0"),
         ("python_versions", "<3.8"),
-        ("transitive_python_versions", "<3.8"),
         ("marker", "sys_platform == 'linux'"),
         ("transitive_marker", "sys_platform == 'linux'"),
     ],
@@ -396,8 +407,9 @@ def test_eq(dependency1: Dependency, dependency2: Dependency, expected: bool) ->
 def test_mutable_attributes_not_in_hash(attr_name: str, value: str) -> None:
     dependency = Dependency("foo", "^1.2.3")
     ref_hash = hash(dependency)
-
     ref_value = getattr(dependency, attr_name)
+
     setattr(dependency, attr_name, value)
+
     assert value != ref_value
     assert hash(dependency) == ref_hash
